﻿class MainHandler
{
    private canvasHtmlElement: HTMLCanvasElement;
    private cvsContext: CanvasRenderingContext2D;
    private crossProductLabelElement: HTMLElement;
    private dotProductLabelElement: HTMLElement;

    private mp1: MovablePoint;
    private mp2: MovablePoint;
    private mp3: MovablePoint;
    private mps: Array<MovablePoint>;

    constructor(cvs: HTMLCanvasElement)
    {
        this.canvasHtmlElement = cvs;
        this.cvsContext = <CanvasRenderingContext2D>this.canvasHtmlElement.getContext("2d");

        this.mp1 = new MovablePoint(this.cvsContext, 50, 250, 1);
        this.mp2 = new MovablePoint(this.cvsContext, 250, 100, 2);
        this.mp3 = new MovablePoint(this.cvsContext, 300, 250, 3);

        this.mps = new Array<MovablePoint>();
        this.mps.push(this.mp1);
        this.mps.push(this.mp2);
        this.mps.push(this.mp3);

        this.canvasHtmlElement.addEventListener("mousedown", this.mouseDown, false);
        this.canvasHtmlElement.addEventListener("mouseup", this.mouseUp, false);
        this.canvasHtmlElement.addEventListener("mousemove", this.mouseMove, false);

        /* get HTML elements and link them to actions */
        this.InitGuiElements();

        this.paint(0);
    }

    private InitGuiElements = (): void =>
    {
        this.crossProductLabelElement = document.getElementById('crossProductLabel');
        this.dotProductLabelElement = document.getElementById('dotProductLabel');
    }

    private paint = (time: number): void =>
    {
        window.requestAnimationFrame(this.paint);

        /* calculate the vectors between v2, v1 and v3, v1 */
        let vec1x = this.mp2.getX() - this.mp1.getX();
        let vec1y = this.mp2.getY() - this.mp1.getY();
        let vec2x = this.mp3.getX() - this.mp1.getX();
        let vec2y = this.mp3.getY() - this.mp1.getY();

        /* note: cross product is the same as the perp-dotproduct */
        let crossProduct = vec1x * vec2y - vec1y * vec2x;
        let dotProduct = vec1x * vec2x + vec1y * vec2y;

        this.cvsContext.save();

        /* fill background */
        this.cvsContext.fillStyle = "#EEBC74";
        this.cvsContext.fillRect(0, 0, this.canvasHtmlElement.width, this.canvasHtmlElement.height);

        /* draw lines */
        this.cvsContext.strokeStyle = "#646464";
        this.cvsContext.beginPath();
        this.cvsContext.lineWidth = 1;
        this.cvsContext.moveTo(this.mp1.getX(), this.mp1.getY());
        this.cvsContext.lineTo(this.mp2.getX(), this.mp2.getY());
        this.cvsContext.stroke();

        this.cvsContext.beginPath();
        this.cvsContext.moveTo(this.mp1.getX(), this.mp1.getY());
        this.cvsContext.lineTo(this.mp3.getX(), this.mp3.getY());
        this.cvsContext.stroke();

        /* draw line to complete parallelogram */
        this.cvsContext.strokeStyle = "#CC7D5D";
        this.cvsContext.lineWidth = 0.5;
        this.cvsContext.beginPath();
        this.cvsContext.moveTo(this.mp2.getX(), this.mp2.getY());
        this.cvsContext.lineTo(this.mp2.getX() + vec2x, this.mp2.getY() + vec2y);
        this.cvsContext.stroke();

        this.cvsContext.beginPath();
        this.cvsContext.moveTo(this.mp3.getX(), this.mp3.getY());
        this.cvsContext.lineTo(this.mp3.getX() + vec1x, this.mp3.getY() + vec1y);
        this.cvsContext.stroke();

        /* draw points */
        this.cvsContext.strokeStyle = "#000000";
        this.mp1.draw();
        this.mp2.draw();
        this.mp3.draw();

        /* print values */
        //this.cvsContext.font = '12px Roboto';
        //this.cvsContext.strokeStyle = "#FF2222";
        //this.cvsContext.strokeText("CrossProduct: " + crossProduct.toString(), 10, 20);
        //this.cvsContext.strokeText("DotProduct: " + dotProduct.toString(), 10, 50);

        this.cvsContext.restore();

        this.crossProductLabelElement.innerHTML = crossProduct.toString();
        this.dotProductLabelElement.innerHTML = dotProduct.toString();
    }

    public mouseDown = (event: MouseEvent): void =>
    {
        let cursorPosX: number = event.x - this.canvasHtmlElement.offsetLeft;
        let cursorPosY: number = event.y - this.canvasHtmlElement.offsetTop;

        for (let i = 0; i < this.mps.length; i++) 
        {
            if (this.mps[i].startDragging(cursorPosX, cursorPosY)) 
            {
                this.canvasHtmlElement.style.cursor = "pointer";
                break;
            }
        }
    }

    public mouseUp = (event: MouseEvent): void => 
    {
        for (let i = 0; i < this.mps.length; i++) 
        {
            this.mps[i].stopDragging();
        }

        /* change cursor back to normal */
        this.canvasHtmlElement.style.cursor = "default";
    }

    public mouseMove = (event: MouseEvent): void => 
    {
        var cursorPosX: number = event.x - this.canvasHtmlElement.offsetLeft;
        var cursorPosY: number = event.y - this.canvasHtmlElement.offsetTop;

        var pointHit: boolean = false;

        /* inform each point about new cursor position */
        for (let i = 0; i < this.mps.length; i++) 
        {
            this.mps[i].cursorMoved(cursorPosX, cursorPosY);
            /* if cursor is over point, change cursor to hand */
            if (this.mps[i].isCursorOnPoint(cursorPosX, cursorPosY)) 
            {
                this.canvasHtmlElement.style.cursor = "pointer";
                pointHit = true;
            }
        }

        /* if cursor is over no point, use normal cursor */
        if (!pointHit)
        {
            this.canvasHtmlElement.style.cursor = "default";
        }
    }

}

window.onload = () => {
    //var el = document.getElementById('content');
    //var greeter = new Greeter(el);
    //greeter.start();

    var canvas = <HTMLCanvasElement>document.getElementById('cvs');
    let mainHandler = new MainHandler(canvas);
};